"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
from logging import Logger
from json.decoder import JSONDecodeError
from Setter.Common.Boards import Boards
from Setter.Common.Command import Command
from Setter.Common.ControllerPins import ControllerPins
from Setter.Common.FileManager import FileManager
from Setter.Exceptions.BackupException import BackupException
from Setter.Exceptions.BoardConfigNotFoundException import BoardConfigNotFoundException
from Setter.Exceptions.BoardNotFoundException import BoardNotFoundException
from Setter.Exceptions.IOErrorException import IOErrorException
from Setter.Exceptions.JSONErrorException import JSONErrorException
from Setter.Logging import Logging


class ControllerConfig:
	"""
	IQRF Gateway Controller configuration class.

	Methods
	-------
	backup() -> None:
		Backup controller configuration.
	restore() -> None:
		Restore controller configuration from backup.
	configure(product: str) -> None:
		Configure controller for specified product
	"""

	__backup_path: str = '/tmp/iqrfControllerConfig.json'
	__config_path: str = '/etc/iqrf-gateway-controller/config.json'
	__name: str = 'IQRF Gateway Controller'
	__logger: Logger = Logging.get_logger()

	@classmethod
	def backup(cls) -> None:
		"""
		Backup controller configuration.

		Current configuration is stored to tmp.

		Returns
		-------
		None

		Raises
		------
		BackupException
			raised if configuration backup failed
		"""
		cls.__logger.info('Backing up %s to tmp', cls.__name)
		command = f'cp -p {cls.__config_path} {cls.__backup_path}'
		result = Command.execute(command)
		if result.returncode != 0:
			error_str = f'Failed to backup {cls.__name} configuration: {result.stderr}.'
			cls.__logger.error(error_str)
			raise BackupException(error_str)

	@classmethod
	def restore(cls) -> None:
		"""
		Restore controller configuration from backup.

		Backed up configuration from tmp is used to restore controller configuration.

		Returns
		-------
		None
		"""
		cls.__logger.info('Restoring %s from tmp', cls.__name)
		command = f'cp -pf {cls.__backup_path} {cls.__config_path}'
		result = Command.execute(command)
		if result.returncode != 0:
			error_str = f'Failed to restore {cls.__name} configuration: {result.stderr}.'
			cls.__logger.error(error_str)
		FileManager.remove_tree(cls.__backup_path)

	@classmethod
	def configure(cls, image: str, product: str) -> None:
		"""
		Configure controller for specified product.

		Retrieves controller configuration for specified product and stores configuration.

		Parameters
		----------
		product: str
			product identified

		Returns
		-------
		None

		Raises
		------
		IOErrorException
			raised if configuration file does not exist
		JSONErrorException
			raised if configuration file has invalid json syntax
		"""
		cls.__logger.info('Configuring %s', cls.__name)
		try:
			pin_config: ControllerPins = Boards.get_controller_pins(product)
			with open(cls.__config_path, mode='r', encoding='utf-8') as file:
				controller_config = json.load(file)

			if image.startswith('iqaros'):
				controller_config['factoryReset']['iqaros'] = True
			else:
				controller_config['factoryReset']['daemon'] = True
				controller_config['factoryReset']['webapp'] = True
			controller_config['resetButton']['button'] = pin_config.button_pin
			controller_config['statusLed']['greenLed'] = pin_config.ledg_pin
			controller_config['statusLed']['redLed'] = pin_config.ledr_pin
			if pin_config.sda_pin is not None:
				if 'powerOff' not in controller_config:
					controller_config['powerOff'] = {}
				controller_config['powerOff']['sda'] = pin_config.sda_pin
			if pin_config.sck_pin is not None:
				if 'powerOff' not in controller_config:
					controller_config['powerOff'] = {}
				controller_config['powerOff']['sck'] = pin_config.sck_pin

			with open(cls.__config_path, mode='w', encoding='utf-8') as file:
				json.dump(controller_config, file, indent=4)
		except (BoardConfigNotFoundException, BoardNotFoundException):
			return
		except IOError as io_error:
			error_str = Logging.io_error(io_error)
			cls.__logger.error(error_str)
			raise IOErrorException(error_str) from io_error
		except JSONDecodeError as json_error:
			error_str = Logging.json_error(json_error, cls.__config_path)
			cls.__logger.error(error_str)
			raise JSONErrorException(error_str) from json_error
		FileManager.remove_tree(cls.__backup_path)
