"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from logging import Logger
from netifaces import AF_LINK, ifaddresses
from Setter.Exceptions.NetworkException import NetworkException
from Setter.Logging import Logging


class NetworkConfig:
	"""
	Gateway network configuration class.

	Methods
	-------
	get_mac(interface: str) -> str
		Return mac address string.
	"""

	__logger: Logger = Logging.get_logger()

	@classmethod
	def get_mac(cls, interface: str) -> str:
		"""
		Return mac address string.

		Parameters
		----------
		interface: str
			interface name

		Returns
		-------
		str
			mac address string

		Exceptions
		----------
		NetworkException
			raised if interface or mac address record don't exist
		"""
		try:
			addrs = ifaddresses(interface)
			if AF_LINK in addrs:
				return addrs[AF_LINK][0]['addr']
			raise Exception('MAC address record not found.')
		except Exception as error:
			error_str = f'Failed to retrieve MAC address for {interface}: {str(error)}.'
			cls.__logger.error(error_str)
			raise NetworkException(error_str) from error
